// Based on an example kindly provided by George Christman and Lance Java.

package jumpstart.business.domain.examples;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Version;

@Entity
@SuppressWarnings("serial")
public class Category implements Serializable {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(nullable = false)
	private Integer id;

	@Version
	@Column(nullable = false)
	private Integer version;

	private String label;

	/**
	 * Category is in an AGGREGATION relationship with itself. Here are its child categories and parent Category. 
	 * Be careful: cycles are not allowed.
	 */
	
	// Do not cascade REMOVE because this is only an AGGREGATION relationship, not a COMPOSITION relationship.
	@OneToMany(mappedBy = "parent", fetch = FetchType.LAZY)
	private Set<Category> children = new HashSet<Category>();

	@ManyToOne
	@JoinColumn(name = "parentId")
	private Category parent;

	public String toString() {
		final String DIVIDER = ", ";
		
		StringBuilder buf = new StringBuilder();
		buf.append(this.getClass().getSimpleName() + ": ");
		buf.append("[");
		buf.append("id=" + id + DIVIDER);
		buf.append("version=" + version + DIVIDER);
		buf.append("label=" + label);
		buf.append("]");
		return buf.toString();
	}

	// The need for an equals() method is discussed at http://www.hibernate.org/109.html
	
	@Override
	public boolean equals(Object obj) {
		return (obj == this) || (obj instanceof Category) && id != null && id.equals(((Category) obj).getId());
	}

	// The need for a hashCode() method is discussed at http://www.hibernate.org/109.html

	@Override
	public int hashCode() {
		return id == null ? super.hashCode() : id.hashCode();
	}

	public Integer getId() {
		return id;
	}

	public Integer getVersion() {
		return version;
	}

	public Set<Category> getChildren() {
		return children;
	}

	public void setChildren(Set<Category> children) {
		this.children = children;
	}

	public Category getParent() {
		return parent;
	}

	public void setParent(Category parent) {
		this.parent = parent;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}
}
