package jumpstart.web.components.examples.component;

import java.util.Set;

import jumpstart.business.domain.examples.Person;

import org.apache.tapestry5.BindingConstants;
import org.apache.tapestry5.ComponentAction;
import org.apache.tapestry5.EventConstants;
import org.apache.tapestry5.Field;
import org.apache.tapestry5.ValidationTracker;
import org.apache.tapestry5.annotations.Environmental;
import org.apache.tapestry5.annotations.Events;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.services.FormSupport;

@Events(EventConstants.VALIDATE)
public class SelectPersonsField implements Field {

	// Parameters

	@Parameter(required = true, allowNull = false)
	@Property
	@SuppressWarnings("unused")
	private Iterable<Person> persons;

	@Parameter(required = true, allowNull = false, name = "chosen")
	@Property
	private Set<Person> chosenPersons;

	@Parameter(defaultPrefix = BindingConstants.LITERAL, value = "0")
	@Property
	private int min;

	@Parameter(defaultPrefix = BindingConstants.LITERAL, value = "false")
	@Property
	private boolean disabled;

	@Parameter(value = "prop:componentResources.id", defaultPrefix = BindingConstants.LITERAL)
	private String clientId;

	@Parameter(defaultPrefix = BindingConstants.LITERAL)
	private String label;

	// Screen fields

	@Property
	private Person person;

	// Generally useful bits and pieces

	private String controlName;

	@Environmental
	private FormSupport formSupport;

	@Environmental
	private ValidationTracker tracker;

	private static final ProcessSubmission PROCESS_SUBMISSION = new ProcessSubmission();

	// The code

	final void setupRender() {

		// If we are inside a form then add my action Setup to its list of things to do on submit and execute it now.
		// It will also be executed later, on submit, BEFORE the components I contain are processed.

		if (formSupport != null) {
			String controlName = formSupport.allocateControlName(clientId);
			ComponentAction<SelectPersonsField> setup = new Setup(controlName);
			formSupport.storeAndExecute(this, setup);
		}

	}

	private void setup(String controlName) {
		this.controlName = controlName;
	}

	final void afterRender() {

		// If we are inside a form then add my action PROCESS_SUBMISSION to its list of things to do on submit.
		// It will be executed later, on submit, AFTER the components I contain are processed (including their
		// validation).

		if (formSupport != null) {
			formSupport.store(this, PROCESS_SUBMISSION);
		}
	}

	private void processSubmission() {

		// Validate. We ensured above that my sub-components will already have been processed before this.
		// Error if the number of persons chosen is less than specified by the min parameter.

		if (chosenPersons.size() < min) {
			tracker.recordError(this, "You must choose at least " + min + " person(s).");
			return;
		}

	}

	// The Loop component will automatically call this for every row as it is rendered.
	public boolean isPersonChosen() {
		return chosenPersons.contains(person);
	}

	// The Loop component will automatically call this for every row on submit.
	public void setPersonChosen(boolean personChosen) {
		if (personChosen) {
			chosenPersons.add(person);
		}
		else {
			chosenPersons.remove(person);
		}
	}

	@Override
	public String getClientId() {
		return null;
	}

	@Override
	public String getControlName() {
		return controlName;
	}

	@Override
	public String getLabel() {
		return label;
	}

	@Override
	public boolean isDisabled() {
		return disabled;
	}

	@Override
	public boolean isRequired() {
		return true;
	}

	private static class Setup implements ComponentAction<SelectPersonsField> {
		private static final long serialVersionUID = 1L;

		private final String controlName;

		Setup(String controlName) {
			this.controlName = controlName;
		}

		public void execute(SelectPersonsField component) {
			component.setup(controlName);
		}

		@Override
		public String toString() {
			return String.format(this.getClass().getSimpleName() + ".Setup[%s]", controlName);
		}
	}

	private static class ProcessSubmission implements ComponentAction<SelectPersonsField> {
		private static final long serialVersionUID = 1L;

		public void execute(SelectPersonsField component) {
			component.processSubmission();
		}

		@Override
		public String toString() {
			return this.getClass().getSimpleName() + ".ProcessSubmission";
		}
	};

}
