package jumpstart.web.components.together.componentscrud;

import java.util.List;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;

import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Events;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Grid;

/**
 * This component will trigger the following events on its container (which in this example is the page):
 * {@link jumpstart.web.components.examples.component.crud.PersonList#SELECTED}(Long personId).
 */
// @Events is applied to a component solely to document what events it may trigger. It is not checked at runtime.
@Events({ PersonList.SELECTED })
public class PersonList {
	public static final String SELECTED = "selected";

	static private final int MAX_RESULTS = 30;

	// Parameters

	@Parameter
	@Property
	private String partialName;

	@Parameter
	@Property
	private Long highlightPersonId;

	// Screen fields

	@Property
	@SuppressWarnings("unused")
	private List<Person> persons;

	@Property
	private Person person;

	@Component(id = "list")
	@Property
	@SuppressWarnings("unused")
	private Grid list;

	// Generally useful bits and pieces

	@EJB
	private IPersonServiceLocal personService;

	// The code

	// setupRender() is called by Tapestry right before it starts rendering the component.

	void setupRender() {
		persons = personService.findPersons(partialName, MAX_RESULTS);
	}

	boolean onSelected(Long personId) {
		// Return false, which means we haven't handled the event so bubble it up.
		// This method is here solely as documentation, because without this method the event would bubble up anyway.
		return false;
	}

	// Getters

	public String getLinkCSSClass() {
		if (person != null && person.getId().equals(highlightPersonId)) {
			return "active";
		}
		else {
			return "";
		}
	}
}
