package jumpstart.web.pages.examples.select;

import java.util.List;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.web.encoders.examples.PersonEncoder;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.SelectModel;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.SelectModelFactory;

public class EasyObjectSelect {
	static private final int MAX_RESULTS = 30;

	// The activation context

	private Long personId;

	// Screen fields

	@Property
	@SuppressWarnings("unused")
	private SelectModel personsModel;

	@Property
	private Person person;

	private PersonEncoder personEncoder;

	// Generally useful bits and pieces

	@Inject
	private SelectModelFactory selectModelFactory;

	@EJB
	private IPersonServiceLocal personService;

	// The code

	Long onPassivate() {
		return personId;
	}

	void onActivate(EventContext context) {
		if (context.getCount() > 0) {
			personId = context.get(Long.class, 0);
		}
	}

	void onPrepareForRender() {
		// Get all persons - ask business service to find them (from the database)
		List<Person> persons = personService.findPersons(MAX_RESULTS);
		
		if (personId != null) {
			person = findPersonInList(personId, persons);
		}

		personsModel = selectModelFactory.create(persons, "firstName");
	}

	void onValidateFromForm() {
		personId = person == null ? null : person.getId();
	}

	private Person findPersonInList(Long personId, List<Person> persons) {
		for (Person person : persons) {
			if (person.getId().equals(personId)) {
				return person;
			}
		}
		return null;
	}

	public PersonEncoder getPersonEncoder() {
		return personEncoder == null ? new PersonEncoder(personService) : personEncoder;
	}

}
